import { Server, Socket } from 'socket.io';

/**
 * Type alias for a constructor of a Room.
 * @template T The constructor parameters type.
 */
type RoomConstructor<T extends unknown[] = unknown[]> = new (...args: T) => Room;
/**
 * Type for room registration, holding the room type and its constructor.
 * @template RT The room type.
 * @template T The constructor parameters type.
 */
type RoomRegistration<RT, T extends unknown[] = unknown[]> = {
    type: RT;
    constructor: RoomConstructor<T>;
};
/**
 * Represents a server with the ability to manage different types of rooms.
 * @template RT The room type.
 */
declare class CyberCafeServer<RT> {
    io: Server;
    private rooms;
    private roomConstructors;
    /**
     * Initializes a new instance of the CyberCafeServer class.
     */
    constructor(io: Server);
    /**
     * Defines a new room type and its associated constructor.
     * @param registration The room registration object containing the room type and constructor.
     */
    defineRoomType<T extends unknown[]>(registration: RoomRegistration<RT, T>): void;
    /**
     * Creates a new room of the specified type.
     * @param roomType The type of the room to create.
     * @param args The constructor arguments for the room.
     * @returns The newly created room instance.
     * @throws When the room type is unrecognized or the room ID is undefined.
     */
    createRoom<T extends string[]>(roomType: RT, ...args: T): Room;
    /**
     * Retrieves a room by its ID.
     * @param roomId The ID of the room to retrieve.
     * @returns The room instance if found, otherwise undefined.
     */
    getRoom(roomId: string): Room | undefined;
    /**
     * Removes a room from the server by its ID.
     * @param roomId The ID of the room to remove.
     */
    removeRoom(roomId: string): void;
}

/**
 * Represents a room within the CyberCafe server environment.
 * This class provides the functionality to manage users and room state.
 */
declare class Room {
    private namespace;
    users: Map<string, Socket>;
    private state;
    private server;
    /**
     * Constructs a new Room instance.
     * @param namespace The namespace identifier for the room.
     * @param server The instance of the CyberCafeServer that this room belongs to.
     */
    constructor(namespace: string, server: CyberCafeServer<string>);
    /**
     * Merges the provided state object with the current room state and broadcasts the updated state to all users.
     * @param update A partial state object with properties to merge into the current state.
     */
    updateState(update: Record<string, unknown>): void;
    /**
     * Lifecycle hook that is called immediately after the room is created.
     * This method can be overridden by subclasses to perform custom initialization.
     */
    protected onCreate(): void;
    /**
     * Lifecycle hook that is called when a user joins the room.
     * This method can be overridden by subclasses to handle user joining events.
     */
    protected onJoin(): void;
    protected onMessage(socket: Socket): void;
    /**
     * Adds a user to the room and associates them with a socket connection.
     * @param userId The unique identifier for the user.
     * @param socket The socket instance associated with the user.
     */
    addUser(userId: string, socket: Socket): void;
    /**
     * Removes a user from the room based on their unique identifier.
     * @param userId The unique identifier for the user to remove.
     */
    removeUser(userId: string): void;
    /**
     * Sends an event with associated data to all users in the room.
     * @param event The name of the event to broadcast.
     * @param data The data to be sent with the event.
     */
    broadcast(event: string, data: unknown): void;
}

export { CyberCafeServer, Room };
